@@ -114,6 +114,7 @@ enum cmd_and_opt_values { aNull = 0,
     oNoComment,
     oCompletesNeeded,
     oMarginalsNeeded,
+    oMaxCertDepth,
     oLoadExtension,
     oRFC1991,
     oCipherAlgo,
@@ -144,6 +145,8 @@ enum cmd_and_opt_values { aNull = 0,
     oS2KMode,
     oS2KDigest,
     oS2KCipher,
+    oCharset,
+    oNotDashEscaped,
 aTest };
 
 
@@ -220,6 +223,7 @@ static ARGPARSE_OPTS opts[] = {
     { oKeyring, "keyring"   ,2, N_("add this keyring to the list of keyrings")},
     { oSecretKeyring, "secret-keyring" ,2, N_("add this secret keyring to the list")},
     { oDefaultKey, "default-key" ,2, N_("|NAME|use NAME as default secret key")},
+    { oCharset, "charset"   , 2, N_("|NAME| set terminal charset to NAME") },
     { oOptions, "options"   , 2, N_("read options from file")},
 
     { oDebug, "debug"     ,4|16, N_("set debugging flags")},
@@ -228,6 +232,7 @@ static ARGPARSE_OPTS opts[] = {
     { oNoComment, "no-comment", 0,   N_("do not write comment packets")},
     { oCompletesNeeded, "completes-needed", 1, N_("(default is 1)")},
     { oMarginalsNeeded, "marginals-needed", 1, N_("(default is 3)")},
+    { oMaxCertDepth,	"max-cert-depth", 1, "@" },
     { oLoadExtension, "load-extension" ,2, N_("|FILE|load extension module FILE")},
     { oRFC1991, "rfc1991",   0, N_("emulate the mode described in RFC1991")},
     { oS2KMode, "s2k-mode",  1, N_("|N|use passphrase mode N")},
@@ -264,7 +269,6 @@ static ARGPARSE_OPTS opts[] = {
   #endif
   #ifdef IS_G10
     { oKOption, NULL,	 0, "@"},
-    { aEditKey, "edit-sig"  ,0, "@"}, /* alias for edit-key */
     { oPasswdFD, "passphrase-fd",1, "@" },
     { aSignKey, "sign-key"  ,256, "@" }, /* alias for edit-key */
   #endif
@@ -291,10 +295,14 @@ static ARGPARSE_OPTS opts[] = {
     { oRunAsShmCP, "run-as-shm-coprocess", 4, "@" },
     { oSetFilename, "set-filename", 2, "@" },
     { oComment, "comment", 2, "@" },
+    { oNotDashEscaped, "not-dash-escaped", 0, "@" },
 {0} };
 
 
 
+int g10_errors_seen = 0;
+
+
 static int maybe_setuid = 1;
 
 static char *build_list( const char *text,
@@ -488,7 +496,6 @@ main( int argc, char **argv )
     unsigned configlineno;
     int parse_debug = 0;
     int default_config =1;
-    int errors=0;
     int default_keyring = 1;
     int greeting = 1;
     enum cmd_and_opt_values cmd = 0;
@@ -529,6 +536,7 @@ main( int argc, char **argv )
     opt.s2k_cipher_algo = CIPHER_ALGO_BLOWFISH;
     opt.completes_needed = 1;
     opt.marginals_needed = 3;
+    opt.max_cert_depth = 5;
     opt.homedir = getenv("GNUPGHOME");
     if( !opt.homedir || !*opt.homedir ) {
       #ifdef __MINGW32__
@@ -599,7 +607,7 @@ main( int argc, char **argv )
 	if( !configfp ) {
 	    if( default_config ) {
 		if( parse_debug )
-		    log_info(_("note: no default option file '%s'\n"),
+		    log_info(_("NOTE: no default option file '%s'\n"),
 							    configname );
 	    }
 	    else {
@@ -695,6 +703,7 @@ main( int argc, char **argv )
 	  case oNoComment: opt.no_comment=1; break;
 	  case oCompletesNeeded: opt.completes_needed = pargs.r.ret_int; break;
 	  case oMarginalsNeeded: opt.marginals_needed = pargs.r.ret_int; break;
+	  case oMaxCertDepth: opt.max_cert_depth = pargs.r.ret_int; break;
 	  case oTrustDBName: trustdb_name = pargs.r.ret_str; break;
 	  case oDefaultKey: opt.def_secret_key = pargs.r.ret_str; break;
 	  case oNoOptions: break; /* no-options */
@@ -755,8 +764,14 @@ main( int argc, char **argv )
 	  case oNoSecmemWarn:
 	    break;  /* dummies */
 	#endif
+	  case oCharset:
+	    if( set_native_charset( pargs.r.ret_str ) )
+		log_error(_("%s is not a valid character set\n"),
+						    pargs.r.ret_str);
+	    break;
+	  case oNotDashEscaped: opt.not_dash_escaped = 1; break;
 
-	  default : errors++; pargs.err = configfp? 1:2; break;
+	  default : pargs.err = configfp? 1:2; break;
 	}
     }
     if( configfp ) {
@@ -810,9 +825,11 @@ main( int argc, char **argv )
 	log_error(_("completes-needed must be greater than 0\n"));
     if( opt.marginals_needed < 2 )
 	log_error(_("marginals-needed must be greater than 1\n"));
+    if( opt.max_cert_depth < 1 || opt.max_cert_depth > 255 )
+	log_error(_("max-cert-depth must be in range 1 to 255\n"));
     switch( opt.s2k_mode ) {
       case 0:
-	log_info(_("note: simple S2K mode (0) is strongly discouraged\n"));
+	log_info(_("NOTE: simple S2K mode (0) is strongly discouraged\n"));
 	break;
       case 1: case 3: break;
       default:
@@ -974,9 +991,17 @@ main( int argc, char **argv )
 
       case aSignKey: /* sign the key given as argument */
       case aEditKey: /* Edit a key signature */
-	if( argc != 1 )
-	    wrong_args(_("--edit-key username"));
-	keyedit_menu(fname, locusr );
+	if( !argc )
+	    wrong_args(_("--edit-key username [commands]"));
+	if( argc > 1 ) {
+	    sl = NULL;
+	    for( argc--, argv++ ; argc; argc--, argv++ )
+		append_to_strlist( &sl, *argv );
+	    keyedit_menu( fname, locusr, sl );
+	    free_strlist(sl);
+	}
+	else
+	    keyedit_menu(fname, locusr, NULL );
 	break;
 
       #endif /* IS_G10 */
@@ -1202,9 +1227,10 @@ main( int argc, char **argv )
 	break;
 
       case aListTrustPath:
-	if( argc != 2 )
-	    wrong_args("--list-trust-path [-- -]<maxdepth> <username>");
-	list_trust_path( atoi(*argv), argv[1] );
+	if( !argc )
+	    wrong_args("--list-trust-path <usernames>");
+	for( ; argc; argc--, argv++ )
+	    list_trust_path( *argv );
 	break;
 
       case aExportOwnerTrust:
@@ -1228,9 +1254,15 @@ main( int argc, char **argv )
 	/* fixme: g10maint should do regular maintenace tasks here */
 	if( argc > 1 )
 	    wrong_args(_("[filename]"));
+	/* Issue some output for the unix newbie */
+	if( !fname && !opt.outfile && isatty( fileno(stdin) )
+		&& isatty( fileno(stdout) ) && isatty( fileno(stderr) ) )
+	    log_info(_("Go ahead and type your message ...\n"));
+
 	if( !(a = iobuf_open(fname)) )
 	    log_error(_("can't open '%s'\n"), print_fname_stdin(fname));
 	else {
+
 	    if( !opt.no_armor ) {
 		if( use_armor_filter( a ) ) {
 		    memset( &afx, 0, sizeof afx);
@@ -1263,7 +1295,8 @@ g10_exit( int rc )
     if( opt.debug )
 	secmem_dump_stats();
     secmem_term();
-    rc = rc? rc : log_get_errorcount(0)? 2:0;
+    rc = rc? rc : log_get_errorcount(0)? 2 :
+			g10_errors_seen? 1 : 0;
     /*write_status( STATUS_LEAVE );*/
     exit(rc );
 }
