@@ -160,7 +160,8 @@ cache_public_key( PKT_public_key *pk )
     if( pk_cache_entries >= MAX_PK_CACHE_ENTRIES ) {
 	/* fixme: use another algorithm to free some cache slots */
 	pk_cache_disabled=1;
-	log_info("too many entries in pk cache - disabled\n");
+	if( opt.verbose )
+	    log_info(_("too many entries in pk cache - disabled\n"));
 	return;
     }
     pk_cache_entries++;
@@ -258,7 +259,8 @@ get_pubkey( PKT_public_key *pk, u32 *keyid )
 	;
     else if( ++unk_cache_entries > MAX_UNK_CACHE_ENTRIES ) {
 	unk_cache_disabled = 1;
-	log_info("too many entries in unk cache - disabled\n");
+	if( opt.verbose )
+	    log_info(_("too many entries in unk cache - disabled\n"));
     }
     else {
 	keyid_list_t kl;
@@ -721,7 +723,7 @@ merge_one_pk_and_selfsig( KBNODE keyblock, KBNODE knode )
 	     */
 	    const byte *p;
 	    p = parse_sig_subpkt( sig->hashed_data, SIGSUBPKT_KEY_EXPIRE, NULL );
-	    pk->expiredate = p? buffer_to_u32(p):0;
+	    pk->expiredate = p? pk->timestamp + buffer_to_u32(p):0;
 	    /* fixme: add usage etc. to pk */
 	    break;
 	}
@@ -739,7 +741,7 @@ merge_keys_and_selfsig( KBNODE keyblock )
     PKT_secret_key *sk = NULL;
     PKT_signature *sig;
     KBNODE k;
-    u32 kid[2];
+    u32 kid[2] = { 0, 0 };
 
     for(k=keyblock; k; k = k->next ) {
 	if( k->pkt->pkttype == PKT_PUBLIC_KEY
@@ -747,7 +749,7 @@ merge_keys_and_selfsig( KBNODE keyblock )
 	    pk = k->pkt->pkt.public_key; sk = NULL;
 	    if( pk->version < 4 )
 		pk = NULL; /* not needed for old keys */
-	    else
+	    else if( k->pkt->pkttype == PKT_PUBLIC_KEY )
 		keyid_from_pk( pk, kid );
 	}
 	else if( k->pkt->pkttype == PKT_SECRET_KEY
@@ -755,7 +757,7 @@ merge_keys_and_selfsig( KBNODE keyblock )
 	    pk = NULL; sk = k->pkt->pkt.secret_key;
 	    if( sk->version < 4 )
 		sk = NULL;
-	    else
+	    else if( k->pkt->pkttype == PKT_SECRET_KEY )
 		keyid_from_sk( sk, kid );
 	}
 	else if( (pk || sk ) && k->pkt->pkttype == PKT_SIGNATURE
@@ -770,12 +772,12 @@ merge_keys_and_selfsig( KBNODE keyblock )
 	    const byte *p;
 	    p = parse_sig_subpkt( sig->hashed_data, SIGSUBPKT_KEY_EXPIRE, NULL );
 	    if( pk ) {
-		pk->expiredate = p? buffer_to_u32(p):0;
+		pk->expiredate = p? pk->timestamp + buffer_to_u32(p):0;
 		/* fixme: add usage etc. */
 		pk = NULL; /* use only the first self signature */
 	    }
 	    else {
-		sk->expiredate = p? buffer_to_u32(p):0;
+		sk->expiredate = p? sk->timestamp + buffer_to_u32(p):0;
 		sk = NULL; /* use only the first self signature */
 	    }
 	}
@@ -937,12 +939,29 @@ finish_lookup( KBNODE keyblock, PKT_public_key *pk, KBNODE k, byte *namehash,
 	    /* if the usage is not correct, try to use a subkey */
 	    KBNODE save_k = k;
 
-	    for( ; k; k = k->next ) {
-		if( k->pkt->pkttype == PKT_PUBLIC_SUBKEY
-		    && !check_pubkey_algo2(
-			    k->pkt->pkt.public_key->pubkey_algo,
-					     pk->pubkey_usage ) )
-		    break;
+	    k = NULL;
+	    /* kludge for pgp 5: which doesn't accept type 20:
+	     * try to use a type 16 subkey instead */
+	    if( pk->pubkey_usage == PUBKEY_USAGE_ENC ) {
+		for( k = save_k; k; k = k->next ) {
+		    if( k->pkt->pkttype == PKT_PUBLIC_SUBKEY
+			&& k->pkt->pkt.public_key->pubkey_algo
+			    == PUBKEY_ALGO_ELGAMAL_E
+			&& !check_pubkey_algo2(
+				k->pkt->pkt.public_key->pubkey_algo,
+						 pk->pubkey_usage ) )
+			break;
+		}
+	    }
+
+	    if( !k ) {
+		for(k = save_k ; k; k = k->next ) {
+		    if( k->pkt->pkttype == PKT_PUBLIC_SUBKEY
+			&& !check_pubkey_algo2(
+				k->pkt->pkt.public_key->pubkey_algo,
+						 pk->pubkey_usage ) )
+			break;
+		}
 	    }
 	    if( !k )
 		k = save_k;
