@@ -38,6 +38,8 @@ typedef struct {
 
 
 /*-- g10.c --*/
+extern int g10_errors_seen;
+
 #if __GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ >= 5 )
   void g10_exit(int rc) __attribute__ ((noreturn));
 #else
@@ -75,7 +77,7 @@ int check_key_signature( KBNODE root, KBNODE node, int *is_selfsig );
 int delete_key( const char *username, int secure );
 
 /*-- keyedit.c --*/
-void keyedit_menu( const char *username, STRLIST locusr );
+void keyedit_menu( const char *username, STRLIST locusr, STRLIST cmds );
 
 /*-- keygen.c --*/
 u32 ask_expiredate(void);
