@@ -49,6 +49,7 @@ struct {
     int no_comment;
     int marginals_needed;
     int completes_needed;
+    int max_cert_depth;
     const char *homedir;
     int skip_verify;
     int compress_keys;
@@ -63,6 +64,7 @@ struct {
     int s2k_mode;
     int s2k_digest_algo;
     int s2k_cipher_algo;
+    int not_dash_escaped;
 } opt;
 
 
