@@ -274,12 +274,13 @@ add_keyblock_resource( const char *url, int force, int secret )
 
 	    iobuf = iobuf_create( filename );
 	    if( !iobuf ) {
-		log_error("%s: can't create keyring: %s\n", filename, strerror(errno));
+		log_error(_("%s: can't create keyring: %s\n"),
+					    filename, strerror(errno));
 		rc = G10ERR_OPEN_FILE;
 		goto leave;
 	    }
 	    else
-		log_info("%s: keyring created\n", filename );
+		log_info(_("%s: keyring created\n"), filename );
 	}
       #ifdef __MINGW32__
 	/* must close it again */
@@ -512,10 +513,12 @@ locate_keyblock_by_fpr( KBPOS *kbpos, const byte *fpr, int fprlen, int secret )
 	if( rentry->used && !rentry->secret == !secret ) {
 	    kbpos->rt = rentry->rt;
 	    switch( rentry->rt ) {
+	     #ifdef HAVE_LIBGDBM
 	      case rt_GDBM:
 		any = 1;
 		rc = do_gdbm_locate( rentry->dbf, kbpos, fpr, fprlen );
 		break;
+	     #endif
 	      default:
 		rc = G10ERR_UNSUPPORTED;
 		break;
@@ -551,10 +554,12 @@ locate_keyblock_by_keyid( KBPOS *kbpos, u32 *keyid, int shortkid, int secret )
 	if( rentry->used && !rentry->secret == !secret ) {
 	    kbpos->rt = rentry->rt;
 	    switch( rentry->rt ) {
+	     #ifdef HAVE_LIBGDBM
 	      case rt_GDBM:
 		any = 1;
 		rc = do_gdbm_locate_by_keyid( rentry->dbf, kbpos, keyid );
 		break;
+	     #endif
 	      default:
 		rc = G10ERR_UNSUPPORTED;
 		break;
@@ -825,6 +830,127 @@ update_keyblock( KBPOS *kbpos, KBNODE root )
 }
 
 
+
+/****************************************************************
+ ********** Implemenation of a user ID database    **************
+ ****************************************************************/
+#if 0
+/****************
+ * Layout of the user ID db
+ *
+ * This user ID DB provides fast lookup of user ID, but the user ids are
+ * not in any specific order.
+ *
+ * A string "GnuPG user db", a \n.
+ * user ids of one key, delimited by \t,
+ * a # or ^ followed by a 20 byte fingerprint, followed by an \n 
+ * The literal characters =, \n, \t, #, ^ must be replaced by a equal sign
+ * and their hex value.
+ *
+ * (We use Boyer/Moore pattern matching)
+ */
+
+/****************
+ * This compiles pattern to the distance table, the table will be allocate
+ * here and must be freed by using free().
+ * Returns: Ptr to new allocated Table
+ *	    Caller must free the table.
+ */
+
+static size_t *
+compile_bm_table( const byte *pattern, size_t len )
+{
+    ushort *dist;
+    int i;
+
+    dist = m_alloc_clear( 256 * sizeof *dist );
+    for(i=0; i < 256; i++ )
+	dist[i] = len;
+    for(i=0; i < len-1; i++ )
+	dTbl[p[i]] = len-i-1;
+    return dist;
+}
+
+
+
+
+/****************
+ * Search BUF of BUFLEN for pattern P of length PATLEN.
+ * dist is the Boyer/Moore distance table of 256 Elements,
+ * case insensitive search is done if IGNCASE is true (In this case
+ * the distance table has to compiled from uppercase chacaters and
+ * PAT must also be uppercase.
+ * Returns: Prt to maching string in BUF, or NULL if not found.
+ */
+
+static const *
+do_bm_search( const byte *buf, size_t buflen,
+	      const byte *pat, size_t patlen, size_t *dist, int igncase )
+{
+    int i, j, k;
+
+    if( igncase ) {
+	int c, c1;
+
+	for( i = --patlen; i < buflen; i += dist[c1] )
+	    for( j=patlen, k=i, c1=c=toupper(buf[k]); c == pat[j];
+					  j--, k--, c=toupper(buf[k]) ) {
+		if( !j )
+		    return buf+k;
+	    }
+    }
+    else {
+	for( i = --patlen; i < buflen; i += dist[buf[i]] )
+	    for( j=patlen, k=i; buf[k] == pat[j]; j--, k-- ) {
+		if( !j )
+		    return buf+k;
+	    }
+    }
+    return NULL;
+}
+
+
+typedef struct {
+    size_t dist[256];
+} *SCAN_USER_HANDLE;
+
+static SCAN_USER_HANDLE
+scan_user_file_open( const byte *name )
+{
+    SCAN_USER_HANDLE hd;
+    size_t *dist;
+    int i;
+
+    hd = m_alloc_clear( sizeof *hd );
+    dist = hd->dist;
+    /* compile the distance table */
+    for(i=0; i < 256; i++ )
+	dist[i] = len;
+    for(i=0; i < len-1; i++ )
+	dTbl[p[i]] = len-i-1;
+    /* setup other things */
+
+    return hd;
+}
+
+static int
+scan_user_file_close( SCAN_USER_HANDLE hd )
+{
+    m_free( hd );
+}
+
+static int
+scan_user_file_read( SCAN_USER_HANDLE hd, byte *fpr )
+{
+    char record[1000];
+
+    /* read a record */
+
+
+}
+#endif
+
+
 
 /****************************************************************
  ********** Functions which operates on regular keyrings ********
@@ -1049,8 +1175,6 @@ keyring_enum( KBPOS *kbpos, KBNODE *ret_root, int skipsigs )
 	     * a start packet; issue a warning if it is not a comment */
 	    if( !root && pkt->pkttype != PKT_COMMENT
 		      && pkt->pkttype != PKT_OLD_COMMENT ) {
-		log_info("keyring_enum: skipped packet of type %d\n",
-			    pkt->pkttype );
 		break;
 	    }
 	    if( !root || (skipsigs && ( pkt->pkttype == PKT_SIGNATURE
@@ -1108,11 +1232,11 @@ keyring_copy( KBPOS *kbpos, int mode, KBNODE root )
 	/* insert: create a new file */
 	newfp = iobuf_create( rentry->fname );
 	if( !newfp ) {
-	    log_error("%s: can't create: %s\n", rentry->fname, strerror(errno));
+	    log_error(_("%s: can't create: %s\n"), rentry->fname, strerror(errno));
 	    return G10ERR_OPEN_FILE;
 	}
 	else
-	    log_info("%s: keyring created\n", rentry->fname );
+	    log_info(_("%s: keyring created\n"), rentry->fname );
 
 	kbctx=NULL;
 	while( (node = walk_kbnode( root, &kbctx, 0 )) ) {
@@ -1285,7 +1409,7 @@ keyring_copy( KBPOS *kbpos, int mode, KBNODE root )
 	rc = G10ERR_RENAME_FILE;
 	if( rentry->secret ) {
 	    log_info(_(
-		"Warning: 2 files with confidential information exists.\n"));
+		"WARNING: 2 files with confidential information exists.\n"));
 	    log_info(_("%s is the unchanged one\n"), rentry->fname );
 	    log_info(_("%s is the new one\n"), tmpfname );
 	    log_info(_("Please fix this possible security flaw\n"));
@@ -1469,7 +1593,7 @@ do_gdbm_locate_by_keyid( GDBM_FILE dbf, KBPOS *kbpos, u32 *keyid )
 	return G10ERR_INV_KEYRING;
     }
     if( content.dsize > 21 )
-	log_info("gdbm_fetch: warning: more than one fingerprint\n" );
+	log_info("gdbm_fetch: WARNING: more than one fingerprint\n" );
 
     rc = do_gdbm_locate( dbf, kbpos, content.dptr+1, 20 );
     free( content.dptr ); /* can't use m_free() here */
