@@ -45,6 +45,7 @@ typedef struct {
 	 int inarg;
 	 int stopped;
 	 const char *last;
+	 void *aliases;
      } internal;	    /* DO NOT CHANGE */
 } ARGPARSE_ARGS;
 
@@ -136,6 +137,7 @@ const char *print_fname_stdout( const char *s );
 /*-- miscutil.c --*/
 u32 make_timestamp(void);
 u32 add_days_to_timestamp( u32 stamp, u16 days );
+const char *strtimevalue( u32 stamp );
 const char *strtimestamp( u32 stamp ); /* GMT */
 const char *asctimestamp( u32 stamp ); /* localized */
 void print_string( FILE *fp, byte *p, size_t n, int delim );
@@ -152,8 +154,10 @@ const char *memistr( const char *buf, size_t buflen, const char *sub );
 char *mem2str( char *, const void *, size_t);
 char *trim_spaces( char *string );
 int string_count_chr( const char *string, int c );
+int set_native_charset( const char *newset );
 char *native_to_utf8( const char *string );
 char *utf8_to_native( const char *string );
+int  check_utf8_string( const char *string );
 
 #define stricmp(a,b) strcasecmp((a),(b))
 
