@@ -27,6 +27,28 @@
 #include "memory.h"
 
 
+static int use_koi8 = 0;
+static ushort koi82unicode[128] = {
+    0x2500,0x2502,0x250c,0x2510,0x2514,0x2518,0x251c,0x2524,
+    0x252c,0x2534,0x253c,0x2580,0x2584,0x2588,0x258c,0x2590,
+    0x2591,0x2592,0x2593,0x2320,0x25a0,0x2219,0x221a,0x2248,
+    0x2264,0x2265,0x00a0,0x2321,0x00b0,0x00b2,0x00b7,0x00f7,
+    0x2550,0x2551,0x2552,0x0451,0x2553,0x2554,0x2555,0x2556,
+    0x2557,0x2558,0x2559,0x255a,0x255b,0x255c,0x255d,0x255e,
+    0x255f,0x2560,0x2561,0x0401,0x2562,0x2563,0x2564,0x2565,
+    0x2566,0x2567,0x2568,0x2569,0x256a,0x256b,0x256c,0x00a9,
+    0x044e,0x0430,0x0431,0x0446,0x0434,0x0435,0x0444,0x0433,
+    0x0445,0x0438,0x0439,0x043a,0x043b,0x043c,0x043d,0x043e,
+    0x043f,0x044f,0x0440,0x0441,0x0442,0x0443,0x0436,0x0432,
+    0x044c,0x044b,0x0437,0x0448,0x044d,0x0449,0x0447,0x044a,
+    0x042e,0x0410,0x0411,0x0426,0x0414,0x0415,0x0424,0x0413,
+    0x0425,0x0418,0x0419,0x041a,0x041b,0x041c,0x041d,0x041e,
+    0x041f,0x042f,0x0420,0x0421,0x0422,0x0423,0x0416,0x0412,
+    0x042c,0x042b,0x0417,0x0428,0x042d,0x0429,0x0427,0x042a
+};
+
+
+
 void
 free_strlist( STRLIST sl )
 {
@@ -180,10 +202,21 @@ string_count_chr( const char *string, int c )
 }
 
 
+int
+set_native_charset( const char *newset )
+{
+    if( !stricmp( newset, "iso-8859-1" ) )
+	use_koi8 = 0;
+    else if( !stricmp( newset, "koi8-r" ) )
+	use_koi8 = 1;
+    else
+	return G10ERR_GENERAL;
+    return 0;
+}
+
 /****************
  * Convert string, which is in native encoding to UTF8 and return the
  * new allocated UTF8 string.
- * This code assumes that native is iso-8859-1.
  */
 char *
 native_to_utf8( const char *string )
@@ -193,21 +226,48 @@ native_to_utf8( const char *string )
     byte *p;
     size_t length=0;
 
-    for(s=string; *s; s++ ) {
-	length++;
-	if( *s & 0x80 )
+    if( use_koi8 ) {
+	for(s=string; *s; s++ ) {
 	    length++;
-    }
-    buffer = m_alloc( length + 1 );
-    for(p=buffer, s=string; *s; s++ ) {
-	if( *s & 0x80 ) {
-	    *p++ = 0xc0 | ((*s >> 6) & 3);
-	    *p++ = 0x80 | ( *s & 0x3f );
+	    if( *s & 0x80 )
+		length += 2; /* we may need 3 bytes */
 	}
-	else
-	    *p++ = *s;
+	buffer = m_alloc( length + 1 );
+	for(p=buffer, s=string; *s; s++ ) {
+	    if( *s & 0x80 ) {
+		ushort val = koi82unicode[ *s & 0x7f ];
+		if( val < 0x0800 ) {
+		    *p++ = 0xc0 | ( (val >> 6) & 0x1f );
+		    *p++ = 0x80 | (  val & 0x3f );
+		}
+		else {
+		    *p++ = 0xe0 | ( (val >> 12) & 0x0f );
+		    *p++ = 0x80 | ( (val >>  6) & 0x3f );
+		    *p++ = 0x80 | (  val & 0x3f );
+		}
+	    }
+	    else
+		*p++ = *s;
+	}
+	*p = 0;
+    }
+    else {
+	for(s=string; *s; s++ ) {
+	    length++;
+	    if( *s & 0x80 )
+		length++;
+	}
+	buffer = m_alloc( length + 1 );
+	for(p=buffer, s=string; *s; s++ ) {
+	    if( *s & 0x80 ) {
+		*p++ = 0xc0 | ((*s >> 6) & 3);
+		*p++ = 0x80 | ( *s & 0x3f );
+	    }
+	    else
+		*p++ = *s;
+	}
+	*p = 0;
     }
-    *p = 0;
     return buffer;
 }
 
@@ -215,13 +275,73 @@ native_to_utf8( const char *string )
 /****************
  * Convert string, which is in UTF8 to native encoding.  Replace
  * illegal encodings by some "\xnn".
- * This code assumes that native is iso-8859-1.
  */
 char *
 utf8_to_native( const char *string )
 {
-    /* FIXME: Not yet done */
-    return m_strdup(string);
+  #if 0
+    const byte *s;
+    size_t n;
+    byte *buffer, *p;
+
+    /* quick check whether we actually have characters with bit 8 set */
+    for( s=string; *s; s++ )
+	if( *s & 0x80 )
+	    break;
+    if( !*s ) /* that is easy */
+	return m_strdup(string);
+
+    /* count the extended utf-8 characters */
+	110x xxxx
+	1110 xxxx
+	1111 0xxx
+    for( n=1, s=string; *s; s++ ) {
+	if( !(*s & 0x80) )
+	    n++;
+	else if( (*s & 0xe0) == 0xc0 )
+	    n += 2;
+	else if( (*s & 0xf0) == 0xe0 )
+	    n += 3;
+	else if( (*s & 0xf8) == 0xf0 )
+	    n += 4;
+	else
+	    n++; /* invalid encoding */
+    }
+
+    buffer = p = m_alloc( n );
+    for( s=string; *s; ) {
+	if( !(*s & 0x80) )
+	    *p++ = *s++;
+	else if( (*s & 0xe0) == 0xc0 ) {
+	    u32 val;
+	    if( (s[1] & 0xc0) != 0x80 )
+		;
+	    val = (*s << 6) | (s[1] & 0x3f);
+	}
+	else if( (*s & 0xf0) == 0xe0 )
+	    n += 3;
+	else if( (*s & 0xf8) == 0xf0 )
+	    n += 4;
+	else
+	    n++; /* invalid encoding */
+    }
+   #endif
+     return m_strdup(string);
+
+}
+
+
+/****************
+ * check whether string is a valid UTF8 string.
+ * Returns 0 = Okay
+ *	   1 = Too short
+ *	   2 = invalid encoding
+ */
+int
+check_utf8_string( const char *string )
+{
+    /*fixme */
+    return 0;
 }
 
 
